@@ -29,7 +29,7 @@
 #
 #########################################################################
 #
-# $Id: install.pl 1543 2006-05-29 17:03:56Z mbr $
+# $Id: install.pl 1597 2006-09-07 03:34:19Z mbr $
 #
 
 use Cwd;
@@ -84,42 +84,44 @@ my $psadCmd      = "${USRSBIN_DIR}/psad";
 ### map perl modules to versions
 my %required_perl_modules = (
     'Unix::Syslog' => {
-        'version' => '0.100',
-        'force-lib-install' => 0,
+        'force-install' => 0,
         'mod-dir' => 'Unix-Syslog'
     },
     'Bit::Vector' => {
-        'version' => '6.3',
-        'force-lib-install' => 0,
+        'force-install' => 0,
         'mod-dir' => 'Bit-Vector'
     },
     'Date::Calc', => {
-        'version' => '5.3',
-        'force-lib-install' => 0,
+        'force-install' => 0,
         'mod-dir' => 'Date-Calc'
     },
     'Net::IPv4Addr' => {
-        'version' => '0.10',
-        'force-lib-install' => 0,
+        'force-install' => 0,
         'mod-dir' => 'Net-IPv4Addr'
     },
     'IPTables::Parse' => {
-        'version' => '0.3',
-        'force-lib-install' => 1,
+        'force-install' => 1,
         'mod-dir' => 'IPTables-Parse'
     },
     'IPTables::ChainMgr' => {
-        'version' => '0.2',
-        'force-lib-install' => 1,
+        'force-install' => 1,
         'mod-dir' => 'IPTables-ChainMgr'
     },
     'Psad' => {
-        'version' => '1.4.1',
-        'force-lib-install' => 1,
+        'force-install' => 1,
         'mod-dir' => 'Psad'
     }
 );
 
+my @cmd_search_paths = qw(
+    /bin
+    /sbin
+    /usr/bin
+    /usr/sbin
+    /usr/local/bin
+    /usr/local/sbin
+);
+
 ### IP regex
 my $ip_re = '(?:\d{1,3}\.){3}\d{1,3}';
 
@@ -161,11 +163,20 @@ my $noarchive   = 0;
 my $uninstall   = 0;
 my $help        = 0;
 my $skip_syslog_test = 0;
+my $skip_module_install   = 0;
 my $cmdline_force_install = 0;
+my $force_path_update = 0;
+my $force_install_re  = '';
 my $syslog_conf = '';
 
+### make Getopts case sensitive
+Getopt::Long::Configure('no_ignore_case');
+
 &usage(1) unless (GetOptions(
     'force-mod-install' => \$cmdline_force_install,  ### force install of all modules
+    'Force-mod-regex=s' => \$force_install_re,  ### force specific mod install with regex
+    'path-update'       => \$force_path_update, ### update command paths
+    'Skip-mod-install'  => \$skip_module_install,
     'no-preserve'       => \$noarchive,   ### Don't preserve existing configs.
     'syslog-conf=s'     => \$syslog_conf, ### specify path to syslog config file.
     'no-syslog-test'    => \$skip_syslog_test,
@@ -339,8 +350,10 @@ sub install() {
     print "\n\n";
 
     ### install perl modules
-    for my $module (keys %required_perl_modules) {
-        &install_perl_module($module);
+    unless ($skip_module_install) {
+        for my $module (keys %required_perl_modules) {
+            &install_perl_module($module);
+        }
     }
 
     &logr("[+] Installing Snort-2.3.3 signatures in $SNORT_DIR\n");
@@ -480,6 +493,12 @@ sub install() {
                 &preserve_old_fw_msg_search();
             }
         }
+
+        if ($force_path_update or not $preserve_rv) {
+            &update_command_paths("$PSAD_CONFDIR/$file")
+                if ($file eq 'psad.conf' or $file eq 'psadwatchd.conf');
+        }
+
         &perms_ownership("${PSAD_CONFDIR}/$file", 0600);
     }
 
@@ -612,7 +631,28 @@ sub install() {
         &logr("[+] Found iptables. Testing syslog configuration:\n");
         ### make sure we actually see packets being logged by
         ### the firewall.
-        &test_syslog_config($syslog_str) if $syslog_str ne 'ulogd';
+        if ($syslog_str ne 'ulogd') {
+            unless (&test_syslog_config($syslog_str)) {
+                if (&query_init_script_restart_syslog()) {
+
+                    my $restarted = 0;
+                    if (-e "$INIT_DIR/sysklogd") {
+                        system "$INIT_DIR/sysklogd restart";
+                        $restarted = 1;
+                    } elsif (-e "$INIT_DIR/syslog") {
+                        system "$INIT_DIR/syslogd restart";
+                        $restarted = 1;
+                    }
+                    ### test syslog config again now that we
+                    ### have restarted syslog via the init script
+                    ### instead of relying on a HUP signal to
+                    ### syslog
+                    if ($restarted) {
+                        &test_syslog_config($syslog_str);
+                    }
+                }
+            }
+        }
     }
 
     ### see if there are any "_CHANGEME_" strings left and give the
@@ -871,22 +911,35 @@ sub stop_psad() {
 sub install_perl_module() {
     my $mod_name = shift;
 
-    die '[*] Missing version key in required_perl_modules hash.'
-        unless defined $required_perl_modules{$mod_name}{'version'};
-    die '[*] Missing force-lib-install key in required_perl_modules hash.'
-        unless defined $required_perl_modules{$mod_name}{'force-lib-install'};
+    die '[*] Missing force-install key in required_perl_modules hash.'
+        unless defined $required_perl_modules{$mod_name}{'force-install'};
     die '[*] Missing mod-dir key in required_perl_modules hash.'
         unless defined $required_perl_modules{$mod_name}{'mod-dir'};
 
-    my $version = $required_perl_modules{$mod_name}{'version'};
+    my $version = '(NA)';
+
+    my $mod_dir = $required_perl_modules{$mod_name}{'mod-dir'};
+
+    if (-e "$mod_dir/VERSION") {
+        open F, "< $mod_dir/VERSION" or
+            die "[*] Could not open $mod_dir/VERSION: $!";
+        $version = <F>;
+        close F;
+        chomp $version;
+    } else {
+        print "[-] Warning: VERSION file does not exist in $mod_dir\n";
+    }
 
     my $install_module = 0;
 
-    if ($required_perl_modules{$mod_name}{'force-lib-install'}
+    if ($required_perl_modules{$mod_name}{'force-install'}
             or $cmdline_force_install) {
         ### install regardless of whether the module may already be
         ### installed
         $install_module = 1;
+    } elsif ($force_install_re and $force_install_re =~ /$mod_name/) {
+        print "[+] Forcing installation of $mod_name module.\n";
+        $install_module = 1;
     } else {
         if (has_perl_module($mod_name)) {
             print "[+] Module $mod_name is already installed in the ",
@@ -918,7 +971,7 @@ sub install_perl_module() {
         system $Cmds{'make'};
 #        system "$Cmds{'make'} test";
         system "$Cmds{'make'} install";
-        chdir $src_dir;
+        chdir $src_dir or die "[*] Could not chdir $src_dir: $!";
 
         print "\n\n";
     }
@@ -1146,6 +1199,20 @@ sub query_preserve_config() {
     return 0;
 }
 
+sub query_init_script_restart_syslog() {
+    my $ans = '';
+    while ($ans ne 'y' && $ans ne 'n') {
+        &logr("[+] Is it ok to restart the syslog daemon ([y]/n)?  ");
+        $ans = <STDIN>;
+        $ans = 'y' if $ans eq "\n";
+        chomp $ans;
+    }
+    if ($ans eq 'y') {
+        return 1;
+    }
+    return 0;
+}
+
 sub query_preserve_sigs_autodl() {
     my $file = shift;
     my $ans = '';
@@ -1315,7 +1382,7 @@ sub test_syslog_config() {
     if ($uprv) {
         &logr("[-] Could not bring up the loopback interface.\n" .
             "    Hoping the syslog reconfig will work anyway.\n");
-        return;
+        return 0;
     }
 
     ### make sure we can see the loopback interface with
@@ -1325,7 +1392,7 @@ sub test_syslog_config() {
     unless (@if_out) {
         &logr("[-] Could not see the loopback interface with ifconfig.  " .
             "Hoping\n    syslog reconfig will work anyway.\n");
-        return;
+        return 0;
     }
 
     my $lo_ip = '127.0.0.1';
@@ -1342,7 +1409,7 @@ sub test_syslog_config() {
     unless ($found_ip) {
         &logr("[-] The loopback interface does not have an IP.\n" .
             "    Hoping the syslog reconfig will work anyway.\n");
-        return;
+        return 0;
     }
 
     ### remove any "test_DROP" lines from fwdata file and ipt_prefix_ctr
@@ -1366,7 +1433,7 @@ sub test_syslog_config() {
         unless (((system "${USRSBIN_DIR}/kmsgsd")>>8) == 0) {
             &logr("[-] Could not start kmsgsd to test syslog.\n" .
                 "    Send email to Michael Rash (mbr\@cipherdyne.org)\n");
-            return;
+            return 0;
         }
     }
 
@@ -1419,15 +1486,14 @@ sub test_syslog_config() {
     }
 
     if ($start_kmsgsd && -e "${RUNDIR}/kmsgsd.pid") {
-        open PID, "${RUNDIR}/kmsgsd.pid" or return;
+        open PID, "${RUNDIR}/kmsgsd.pid" or return 0;
         my $pid = <PID>;
         close PID;
         chomp $pid;
-        if (kill 0, $pid) {
-            kill 9, $pid;
-        }
+        kill 9, $pid if kill 0, $pid;
     }
-    return;
+    return 1 if $found;
+    return 0;
 }
 
 sub scrub_prefix_ctr() {
@@ -1794,18 +1860,10 @@ sub enable_psad_at_boot() {
 
 ### check paths to commands and attempt to correct if any are wrong.
 sub check_commands() {
-    my @path = qw(
-        /bin
-        /sbin
-        /usr/bin
-        /usr/sbin
-        /usr/local/bin
-        /usr/local/sbin
-    );
     CMD: for my $cmd (keys %Cmds) {
         unless (-x $Cmds{$cmd}) {
             my $found = 0;
-            PATH: for my $dir (@path) {
+            PATH: for my $dir (@cmd_search_paths) {
                 if (-x "${dir}/${cmd}") {
                     $Cmds{$cmd} = "${dir}/${cmd}";
                     $found = 1;
@@ -1899,6 +1957,57 @@ sub has_perl_module() {
     return $@ ? 0 : 1;
 }
 
+sub update_command_paths() {
+    my $file = shift;
+
+    open F, "< $file" or die "[*] Could not open file: $!";
+    my @lines = <F>;
+    close F;
+
+    my @newlines = ();
+    my $new_cmd = 0;
+    for my $line (@lines) {
+        my $found = 0;
+        if ($line =~ /^\s*(\w+)Cmd(\s+)(\S+);/) {
+            my $cmd    = $1;
+            my $spaces = $2;
+            my $path   = $3;
+            unless (-e $path and -x $path) {
+                ### the command is not at this path, try to find it
+                my $cmd_minor_name = $cmd;
+                if ($path =~ m|.*/(\S+)|) {
+                    $cmd_minor_name = $cmd if $cmd ne $1;
+                }
+                DIR: for my $dir (@cmd_search_paths) {
+                    if (-e "$dir/$cmd_minor_name"
+                            and -x "$dir/$cmd_minor_name") {
+                        ### found the command
+                        push @newlines,
+                            "${cmd}Cmd${spaces}${dir}/${cmd_minor_name};\n";
+                        $found   = 1;
+                        $new_cmd = 1;
+                        last DIR;
+                    }
+                }
+                unless ($found) {
+                    &logr("[-] Could not find the path to the $cmd command, " .
+                        "you will need to manually\n    edit the path for " .
+                        "the ${cmd}Cmd variable in $file\n");
+                }
+            }
+        }
+        unless ($found) {
+            push @newlines, $line;
+        }
+    }
+    if ($new_cmd) {
+        open C, "> $file" or die "[*] Could not open file: $!";
+        print C for @newlines;
+        close C;
+    }
+    return;
+}
+
 ### logging subroutine that handles multiple filehandles
 sub logr() {
     my $msg = shift;
@@ -1921,17 +2030,23 @@ sub usage() {
     my $exitcode = shift;
     print <<_HELP_;
 
-Usage: install.pl [-f] [-s <file>] [-u] [--no-syslog-test]
-                  [--no-preserve] [-h]
+Usage: install.pl [-f] [-s <file>] [-u] [--no-syslog-test] [-S] [-p]
+                  [-F] [--no-preserve] [-h]
 
-    -u  --uninstall     - Uninstall psad.
-    -f  --force-mod     - Force all perl modules to be installed
-                          even if some already exist in the system
-                          /usr/lib/perl5 tree.
-    -s  <file>          - Specify path to syslog.conf file.
-    --no-syslog-test    - Skip syslog reconfiguration test.
-    --no-preserve       - Disable preservation of old configs.
-    -h  --help          - Prints this help message.
+    -u,  --uninstall            - Uninstall psad.
+    -f, --force-mod-install     - Force all perl modules to be installed
+                                  even if some already exist in the system
+                                  /usr/lib/perl5 tree.
+    -F, --Force-mod-regex <re>  - Specify a regex to match a module name
+                                  and force the installation of such
+                                  modules.
+    -p, --path-update           - Run path update code regardless of whether
+                                  a previous config is being merged.
+    -S, --Skip-mod-install      - Do not install any perl modules.
+    -s  <file>                  - Specify path to syslog.conf file.
+    --no-syslog-test            - Skip syslog reconfiguration test.
+    --no-preserve               - Disable preservation of old configs.
+    -h  --help                  - Prints this help message.
 
 _HELP_
     exit $exitcode;
